#ifndef __mMatrix_H__
#define __mMatrix_H__

/*! 
    \file mMatrix.h
    \brief 4x4 Column-order Matrix
    \author Douglas H. Cox
*/

//============================================================================
//    REQUIRED HEADERS
//============================================================================
#include "mMath.h"

//============================================================================
//    DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS / CLASS FORWARDS
//============================================================================

//============================================================================
//    FUNCTION PROTOTYPES
//============================================================================

//============================================================================
//    CLASS PROTOTYPES
//============================================================================

//! 4x4 Column-order Matrix
/*! Matrices are meant to be used only when needed by the
    rendering API.  OCSes should be used instead.
    \sa mOCS::getMatrix()
*/
class MMATH_DLL mMatrix
{
public:
  float m[16]; //!< Matrix data in column order.

public:
  //! Default constructor
  /*! No initialization is done here. 
  */
  mMatrix()
  {
  }

  //! Copy constructor
  mMatrix(const mMatrix& src)
  { 
    set(src);
  }

  //! Assignment
  void set(const mMatrix& src);

  //! Assignment
  mMatrix& operator = (const mMatrix& src) 
  { 
    set(src); 
    return *this; 
  }

  //! Accessor
  float& operator [] (int i)
  {
    return m[i];
  }

  //! Accessor
  float operator [] (int i) const
  {
    return m[i];
  }

  //! Creates an identity matrix.
  void makeIdentity();

  //! Multiplies matrix a by matrix b.
  /*! \todo Optimize this. */
  void mult(const mMatrix& a, const mMatrix& b);

};

//****************************************************************************
//**    END HEADER mMatrix.h
//****************************************************************************
#endif // __mMatrix_H__
