//****************************************************************************
//**
//**    mMath.cpp
//**    Common math functions.
//**    (C) 2000 - Douglas H. Cox
//**
//****************************************************************************
//============================================================================
//    HEADERS
//============================================================================
#include "mMath.h"

// The following results in a ~10k drop in size for a release build.
#ifdef NDEBUG
#if _MSC_VER >= 1000
#pragma comment(linker,"/OPT:nowin98")
#endif // _MSC_VER >= 1000
#endif // NDEBUG

//============================================================================
//    PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
//============================================================================

//============================================================================
//    PRIVATE STRUCTURES / UTILITY CLASSES
//============================================================================

//============================================================================
//    PRIVATE DATA
//============================================================================

// Square root table used for fast square root function.
unsigned int mFastSqrtTable[0x10000];

// Keep track of initialized ref count
static int s_initRefCount = 0;

//============================================================================
//    PRIVATE FUNCTIONS
//============================================================================

// ============================================================
// buildSqrtTable - from nVidia's fastmath.cpp
void buildFastSqrtTable()
{
  union uFloatUInt
  {
    float f;
    unsigned int i;
  } s;

  unsigned int i;

  for (i = 0; i <= 0x7FFF; i++)
  {
    // Build a float with the bit pattern i as mantissa
    // and an exponent of 0, stored as 127
    
    s.i = (i << 8) | (0x7F << 23);
    s.f = (float)sqrt(s.f);
    
    // Take the square root then strip the first 7 bits of
    // the mantissa into the table
    
    mFastSqrtTable[i + 0x8000] = (s.i & 0x7FFFFF);
    
    // Repeat the process, this time with an exponent of 1, 
    // stored as 128
    
    s.i = (i << 8) | (0x80 << 23);
    s.f = (float)sqrt(s.f);
    
    mFastSqrtTable[i] = (s.i & 0x7FFFFF);
  }
}

//============================================================================
//    FUNCTIONS
//============================================================================

// ============================================================
// mInitialize
bool mInitialize()
{
  s_initRefCount++;
  if (s_initRefCount > 1)
    return true;

  // Initialize fast square root table.
  buildFastSqrtTable();

  return true;
}

// ============================================================
// mUninitialize
void mUninitialize()
{
  s_initRefCount--;
  if (s_initRefCount > 0)
    return;
  
  // Free up any allocated memory here.
}

//============================================================================
//    CLASS METHODS
//============================================================================

//****************************************************************************
//**    END MODULE mMath.cpp
//****************************************************************************

