//****************************************************************************
//**
//**    mRay.cpp
//**    Ray class.
//**    (C) 2000 - Douglas H. Cox
//**
//****************************************************************************
//============================================================================
//    HEADERS
//============================================================================
#include "mRay.h"

#include "mOCS.h"

//============================================================================
//    PRIVATE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
//============================================================================

//============================================================================
//    PRIVATE STRUCTURES / UTILITY CLASSES
//============================================================================

//============================================================================
//    PRIVATE DATA
//============================================================================

//============================================================================
//    PRIVATE FUNCTIONS
//============================================================================

//============================================================================
//    FUNCTIONS
//============================================================================

//============================================================================
//    CLASS METHODS
//============================================================================

// ============================================================
// mRay::set
void mRay::set(const mVec3& startPoint, const mVec3& endPoint)
{
  pos.set(startPoint);
  dir.sub(endPoint, startPoint);
  len = dir.normalize();
}

// ============================================================
// mRay::operator == 
bool mRay::operator == (const mRay& ray) const
{
  return (len == ray.len &&
          pos == ray.pos &&
          dir == ray.dir);
}

// ============================================================
// mRay::getEndPoint
void mRay::getEndPoint(mVec3& end) const
{
  (len >= 0.0f) ? end.addScaled(pos, dir, len) :
                  end.addScaled(pos, dir, mFloatMax);
}

// ============================================================
// mRay::xformTo
void mRay::xformTo(const mRay& ray, const mOCS& ocs)
{
  pos.xformTo(ray.pos, ocs);
  dir.xformVecTo(ray.dir, ocs);

  if (len < 0.0f)
  {
    if (ocs.type & mOCS::SCALE)
      dir.normalize();
  }
  else
  {
    if (ocs.type & mOCS::SCALE)
    {
      len *= dir.normalize();
    }
  }
}

// ============================================================
// mRay::xformFrom
void mRay::xformFrom(const mRay& ray, const mOCS& ocs)
{
  pos.xformFrom(ray.pos, ocs);
  dir.xformVecFrom(ray.dir, ocs);

  if (len < 0.0f)
  {
    if (ocs.type & mOCS::SCALE)
      dir.normalize();
  }
  else
  {
    if (ocs.type & mOCS::SCALE)
    {
      len *= dir.normalize();
    }
  }
}

//****************************************************************************
//**    END MODULE mRay.cpp
//****************************************************************************

